﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Infragistics.Olap.AdomdClient.AdomdWebService;
using Infragistics.Olap.Data;
using Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008;
using Infragistics.Olap.Xmla;

namespace Infragistics.Olap.AdomdClient
{
    public class AdomdCommand : XmlaCommand
    {
        public AdomdCommand(XmlaConnection connection) : base(connection)
        {
        }

        public override event EventHandler<ExecuteCommandCompletedEventArgs> ExecuteCommandCompleted;

        public override void ExecuteCommandAsync(IQuery query, XmlaConnection connection)
        {
            string commandText = GenerateCommand(query);
            if (this.CancelPending)
            {
                return;
            }

            AdomdWebServiceClient client = new AdomdWebServiceClient();

            client.GetGridDataCompleted += this.ClientGetGridDataCompleted;

            client.GetGridDataAsync(commandText, query);
        }

        private void ClientGetGridDataCompleted(object sender, GetGridDataCompletedEventArgs e)
        {
            IQuery query = e.UserState as IQuery;
            if (query == null)
            {
                return;
            }

            Result result = null;

            if (!e.Cancelled && e.Error == null && e.Result != null && !this.CancelPending)
            {
                    AxisData columnAxis = null;
                    if (query.ColumnAxis != null)
                    {
                        columnAxis = e.Result.Axes[0];
                    }

                    AxisData rowAxis = null;
                    if (query.RowAxis != null)
                    {
                        rowAxis = e.Result.Axes[columnAxis == null ? 0 : 1];
                    }

                    result = GenerateResult(columnAxis, rowAxis, e.Result.Cells);
                
            }

            if (this.ExecuteCommandCompleted != null)
            {
                this.ExecuteCommandCompleted(
                    this,
                    new ExecuteCommandCompletedEventArgs(result, e.Error, e.Cancelled || this.CancelPending, e.UserState));
            }
        }

        private static Result GenerateResult(AxisData columnAxis, AxisData rowAxis, ObservableCollection<CellData> cellsData)
        {
            ResultAxis resultColumnAxis = GenerateAxis(columnAxis);

            ResultAxis resultRowAxis = GenerateAxis(rowAxis);

            ICell[,] cells = new ICell[resultRowAxis.Tuples.Count, resultColumnAxis.Tuples.Count];
            FillCellData(cells, cellsData);

            return new Result(resultRowAxis, resultColumnAxis, cells);
        }

        private static void FillCellData(ICell[,] cells, ObservableCollection<CellData> cellsData)
        {
            if (cellsData == null)
            {
                return;
            }

            int columnsCount = cells.GetLength(1);

            for (int i = 0; i < cellsData.Count; i++)
            {
                CellData cellData = cellsData[i];
                int columnNumber = i % columnsCount;
                int rowNumber = i / columnsCount;

                cells[rowNumber, columnNumber] = new Data.Base.Cell(cellData.Value, cellData.FormattedValue);
            }
        }

        private static ResultAxis GenerateAxis(AxisData axis)
        {
            List<ITuple> tuples = new List<ITuple>();

            if (axis == null)
            {
                List<IMember> members = new List<IMember>();

                Tuple axisTuple = new Tuple(new OlapElementCollection<IMember>(members));
                tuples.Add(axisTuple);
            }
            else
            {
                foreach (PositionData positionData in axis.Positions)
                {
                    List<IMember> members = new List<IMember>();
                    foreach (MemberData memberData in positionData.Members)
                    {
                        Member member = new Member(
                            memberData.Name, 
                            memberData.UniqueName,
                            memberData.Caption,
                            memberData.ParentUniqueName,
                            memberData.LevelName,
                            memberData.LevelDepth,
                            null, // TODO : Find if we can set parent level
                            memberData.ChildCount);

                        members.Add(member);
                    }

                    Tuple axisTuple = new Tuple(new OlapElementCollection<IMember>(members));
                    tuples.Add(axisTuple);
                }
            }

            return new ResultAxis(new ReadOnlyCollection<ITuple>(tuples));
        }
    }
}
