﻿using System;
using System.ComponentModel;
using Infragistics.Olap.Data;
using Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008;

namespace Infragistics.Olap.AdomdClient
{
    public class AdomdConnectionSettings : IConnectionSettings
    {
        #region Static constructor

        /// <summary>
        /// Initializes static members of the <see cref="AdomdConnectionSettings"/> class. 
        /// Registers classes into container.
        /// </summary>
        static AdomdConnectionSettings()
        {
            OlapComposition olapComposition = new OlapComposition();
            olapComposition.AddOlapPart<AdomdViewModel>(typeof(IDataProvider));
            olapComposition.AddOlapPart<AdomdModelProvider>(typeof(IOlapConnection));
            olapComposition.AddOlapPart<AdomdConnection>(typeof(IConnectionSettings));
            olapComposition.AddOlapPart<AdomdConnectionSettings>();

            OlapContainer<IOlapViewModel>.Instance.Register(typeof(AdomdConnectionSettings), olapComposition);

            OlapComposition filterComposition = new OlapComposition();
            filterComposition.AddOlapPart<FilterViewModelBase>(typeof(IDataProvider), typeof(ICube), typeof(IHierarchy), typeof(bool));
            filterComposition.AddOlapPart<AdomdModelProvider>(typeof(IOlapConnection));
            filterComposition.AddOlapPart<AdomdConnection>(typeof(IConnectionSettings));
            filterComposition.AddOlapPart<AdomdConnectionSettings>();
            filterComposition.AddOlapPart<Cube>();
            filterComposition.AddOlapPart<Hierarchy>();

            OlapContainer<IFilterViewModel>.Instance.Register(typeof(AdomdConnectionSettings), filterComposition);

            OlapComposition measureComposition = new OlapComposition();
            measureComposition.AddOlapPart<BaseMeasureViewModel>(typeof(IDataProvider), typeof(ICube), typeof(IMeasure));
            measureComposition.AddOlapPart<AdomdModelProvider>(typeof(IOlapConnection));
            measureComposition.AddOlapPart<AdomdConnection>(typeof(IConnectionSettings));
            measureComposition.AddOlapPart<AdomdConnectionSettings>();
            measureComposition.AddOlapPart<Cube>();
            measureComposition.AddOlapPart<Measure>();

            OlapContainer<IMeasureViewModel>.Instance.Register(typeof(AdomdConnectionSettings), measureComposition);
        }

        #endregion // Static constructor

        #region Properties

        /// <summary>
        /// Gets or sets the server URL. 
        /// </summary>
        /// <example>Microsoft SQL Analysis Server: http://[domain:port]/olap/msmdpump.dll</example>
        /// <value>The server URL.</value>
        [TypeConverter(typeof(UriTypeConverter))]
        public Uri ServerUrl { get; set; }

        /// <summary>
        /// Gets or sets the name of the catalog.
        /// <example>Adventure Works DW</example>
        /// </summary>
        /// <value>The name of the catalog.</value>
        public string CatalogName { get; set; }

        #endregion // Properties
    }
}
